@@ -1,5 +1,5 @@
 var group__paired__modules =
 [
-    [ "G-quadruplexes", "group__gquads.html", "group__gquads" ],
+    [ "G-Quadruplexes", "group__gquads.html", "group__gquads" ],
     [ "gquad.h", "gquad_8h.html", null ]
 ];
\ No newline at end of file
