@@ -18,17 +18,18 @@
 #include <string.h>
 #include <math.h>
 #include <float.h>    /* #defines FLT_MAX ... */
-#include "ViennaRNA/data_structures.h"
-#include "ViennaRNA/utils.h"
-#include "ViennaRNA/energy_par.h"
+#include "ViennaRNA/datastructures/basic.h"
+#include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/params/default.h"
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/PS_dot.h"
+#include "ViennaRNA/plotting/probabilities.h"
 #include "ViennaRNA/part_func.h"
-#include "ViennaRNA/params.h"
-#include "ViennaRNA/loop_energies.h"
+#include "ViennaRNA/params/basic.h"
+#include "ViennaRNA/loops/all.h"
 #include "ViennaRNA/LPfold.h"
 #include "ViennaRNA/Lfold.h"
-
+#include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/part_func_window.h"
 
 /*
  #################################
@@ -83,7 +84,7 @@ typedef void (add_QI5)(FLT_OR_DBL **,
  #################################
  */
 
-#ifdef  VRNA_BACKWARD_COMPAT
+#ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
 
 #ifdef _OPENMP
 #include <omp.h>
@@ -142,9 +143,10 @@ probability_correction(vrna_fold_compound_t *vc,
 
 
 #if 0
-PRIVATE vrna_ep_t *get_deppp(vrna_fold_compound_t *vc,
-                             vrna_ep_t            *pl,
-                             int                  start);
+PRIVATE vrna_ep_t *
+get_deppp(vrna_fold_compound_t  *vc,
+          vrna_ep_t             *pl,
+          int                   start);
 
 
 #endif
@@ -188,6 +190,7 @@ store_bpp_callback(FLT_OR_DBL *pr,
                    void       *data);
 
 
+#if 0
 PRIVATE void
 store_stack_prob_callback(FLT_OR_DBL  *pr,
                           int         size,
@@ -195,6 +198,8 @@ store_stack_prob_callback(FLT_OR_DBL  *pr,
                           void        *data);
 
 
+#endif
+
 PRIVATE void
 print_pU_callback(double        *pU,
                   int           size,
@@ -278,33 +283,6 @@ vrna_pfl_fold(const char  *sequence,
 }
 
 
-PUBLIC void
-vrna_pfl_fold_cb(const char                 *sequence,
-                 int                        window_size,
-                 int                        max_bp_span,
-                 vrna_probs_window_callback *cb,
-                 void                       *data)
-{
-  unsigned int          options;
-  vrna_fold_compound_t  *vc;
-  vrna_md_t             md;
-
-  vrna_md_set_default(&md);       /* get default parameters */
-
-  md.compute_bpp  = 1;            /* turn on base pair probability computations */
-  md.window_size  = window_size;  /* set size of sliding window */
-  md.max_bp_span  = max_bp_span;  /* set maximum base pair span */
-
-  vc = vrna_fold_compound(sequence, &md, VRNA_OPTION_PF | VRNA_OPTION_WINDOW);
-
-  options = VRNA_PROBS_WINDOW_BPP; /* always compute base pair probabilities */
-
-  vrna_probs_window(vc, 0, options, cb, data);
-
-  vrna_fold_compound_free(vc);
-}
-
-
 PUBLIC double **
 vrna_pfl_fold_up(const char *sequence,
                  int        ulength,
@@ -346,34 +324,6 @@ vrna_pfl_fold_up(const char *sequence,
 }
 
 
-PUBLIC void
-vrna_pfl_fold_up_cb(const char                  *sequence,
-                    int                         ulength,
-                    int                         window_size,
-                    int                         max_bp_span,
-                    vrna_probs_window_callback  *cb,
-                    void                        *data)
-{
-  unsigned int          options;
-  vrna_fold_compound_t  *vc;
-  vrna_md_t             md;
-
-  vrna_md_set_default(&md);       /* get default parameters */
-
-  md.compute_bpp  = 1;            /* turn on base pair probability computations */
-  md.window_size  = window_size;  /* set size of sliding window */
-  md.max_bp_span  = max_bp_span;  /* set maximum base pair span */
-
-  vc = vrna_fold_compound(sequence, &md, VRNA_OPTION_PF | VRNA_OPTION_WINDOW);
-
-  options = VRNA_PROBS_WINDOW_UP; /* compute unpaired probabilties */
-
-  vrna_probs_window(vc, ulength, options, cb, data);
-
-  vrna_fold_compound_free(vc);
-}
-
-
 PRIVATE void
 alloc_helper_arrays(vrna_fold_compound_t  *vc,
                     int                   ulength,
@@ -716,7 +666,7 @@ rotate_constraints(vrna_fold_compound_t *fc,
 }
 
 
-PUBLIC void
+PUBLIC int
 vrna_probs_window(vrna_fold_compound_t        *vc,
                   int                         ulength,
                   unsigned int                options,
@@ -732,18 +682,19 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
   vrna_mx_pf_t        *matrices;
   vrna_hc_t           *hc;
   helper_arrays       aux_arrays;
-  vrna_mx_pf_aux_el_t *aux_mx_el;
-  vrna_mx_pf_aux_ml_t *aux_mx_ml;
+  vrna_mx_pf_aux_el_t aux_mx_el;
+  vrna_mx_pf_aux_ml_t aux_mx_ml;
 
   ov    = 0;
   Qmax  = 0;
 
   if ((!vc) || (!cb))
-    return;
+    return 0; /* failure */
 
   if (!vrna_fold_compound_prepare(vc, VRNA_OPTION_PF | VRNA_OPTION_WINDOW)) {
-    vrna_message_warning("vrna_probs_window@LPfold.c: Failed to prepare vrna_fold_compound");
-    return;
+    vrna_message_warning("vrna_probs_window: "
+                         "Failed to prepare vrna_fold_compound");
+    return 0; /* failure */
   }
 
   /* here space for initializing everything */
@@ -792,7 +743,7 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
 
     free_helper_arrays(vc, ulength, &aux_arrays, options);
 
-    return;
+    return 1; /* success */
   }
 
   init_dp_matrices(vc, options);
@@ -807,19 +758,22 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
   /* start recursions */
   for (j = turn + 2; j <= n + winSize; j++) {
     if (j <= n) {
+      vrna_exp_E_ext_fast_update(vc, j, aux_mx_el);
       for (i = j - turn - 1; i >= MAX2(1, (j - winSize + 1)); i--) {
         hc_decompose  = hc->matrix_local[i][j - i];
         qbt1          = 0.;
 
-        /* construction of partition function of segment i,j */
-        /* firstly that given i bound to j : qb(i,j) */
+        /*
+         * construction of partition function of segment i,j
+         * firstly that given i bound to j : qb(i,j)
+         */
         if (hc_decompose) {
           /* process hairpin loop(s) */
           qbt1 += vrna_exp_E_hp_loop(vc, i, j);
           /* process interior loop(s) */
           qbt1 += vrna_exp_E_int_loop(vc, i, j);
           /* process multibranch loop(s) */
-          qbt1 += vrna_exp_E_mb_loop_fast(vc, i, j, aux_mx_ml->qqm1);
+          qbt1 += vrna_exp_E_mb_loop_fast(vc, i, j, aux_mx_ml);
         }
 
         qb[i][j] = qbt1;
@@ -828,10 +782,11 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
         qm[i][j] = vrna_exp_E_ml_fast(vc, i, j, aux_mx_ml);
         if ((options & VRNA_PROBS_WINDOW_UP) && (ulength > 0)) {
           /* new qm2 computation done here */
+          const FLT_OR_DBL *qqm = vrna_exp_E_ml_fast_qqm(aux_mx_ml);
           temp = 0.0;
           for (k = i + 1; k <= j; k++)
             temp += qm[i][k - 1] *
-                    aux_mx_ml->qqm[k];
+                    qqm[k];
           qm2[i][j] = temp;
         }
 
@@ -840,13 +795,28 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
 
         if (temp > Qmax) {
           Qmax = temp;
-          if (Qmax > max_real / 10.)
-            vrna_message_warning("Q close to overflow: %d %d %g\n", i, j, temp);
+          if (Qmax > max_real / 10.) {
+            vrna_message_warning("vrna_probs_window: "
+                                 "Q close to overflow: %d %d %g\n",
+                                 i,
+                                 j,
+                                 temp);
+          }
         }
 
-        if (temp >= max_real)
-          vrna_message_error("overflow in pf_fold while calculating q[%d,%d]\n"
-                             "use larger pf_scale", i, j);
+        if (temp >= max_real) {
+          vrna_message_warning("vrna_probs_window: "
+                               "overflow while computing partition function for segment q[%d,%d]\n"
+                               "use larger pf_scale",
+                               i,
+                               j);
+
+          vrna_exp_E_ml_fast_free(aux_mx_ml);
+          vrna_exp_E_ext_fast_free(aux_mx_el);
+          free_helper_arrays(vc, ulength, &aux_arrays, options);
+
+          return 0; /* failure */
+        }
       } /* end for i */
 
       /*
@@ -879,8 +849,8 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
       }
 
       /* rotate auxiliary arrays */
-      vrna_exp_E_ext_fast_rotate(vc, aux_mx_el);
-      vrna_exp_E_ml_fast_rotate(vc, aux_mx_ml);
+      vrna_exp_E_ext_fast_rotate(aux_mx_el);
+      vrna_exp_E_ml_fast_rotate(aux_mx_ml);
     }
 
     if (j > winSize) {
@@ -956,19 +926,24 @@ vrna_probs_window(vrna_fold_compound_t        *vc,
     }
   }
 
-  if (ov > 0)
-    vrna_message_warning("%d overflows occurred while backtracking;\n"
+  if (ov > 0) {
+    vrna_message_warning("vrna_probs_window: "
+                         "%d overflows occurred while backtracking;\n"
                          "you might try a smaller pf_scale than %g\n",
-                         ov, pf_params->pf_scale);
+                         ov,
+                         pf_params->pf_scale);
+  }
 
   free_dp_matrices(vc, options);
   free_helper_arrays(vc, ulength, &aux_arrays, options);
 
   /* free memory occupied by auxiliary arrays for fast exterior/multibranch loops */
-  vrna_exp_E_ml_fast_free(vc, aux_mx_ml);
-  vrna_exp_E_ext_fast_free(vc, aux_mx_el);
+  vrna_exp_E_ml_fast_free(aux_mx_ml);
+  vrna_exp_E_ext_fast_free(aux_mx_el);
 
   free(Fwindow);
+
+  return 1; /* success */
 }
 
 
@@ -1106,8 +1081,10 @@ compute_probs(vrna_fold_compound_t        *vc,
 
   /* start recursion */
 
-  /* i=j-winSize; */
-  /* initialize multiloopfs */
+  /*
+   * i=j-winSize;
+   * initialize multiloopfs
+   */
   for (k = j - winSize; k <= MIN2(n, j); k++) {
     prml[k]   = 0;
     prm_l[k]  = 0;
@@ -1118,11 +1095,8 @@ compute_probs(vrna_fold_compound_t        *vc,
 
   for (l = k + turn + 1; l <= MIN2(n, k + winSize - 1); l++) {
     int a;
-    pR[k][l] = 0;  /* set zero at start */
-
-    type = ptype[k][l];
-    if (type == 0)
-      type = 7;
+    pR[k][l]  = 0; /* set zero at start */
+    type      = vrna_get_ptype_window(k, l + k, ptype);
 
     if (qb[k][l] == 0)
       continue;
@@ -1146,21 +1120,16 @@ compute_probs(vrna_fold_compound_t        *vc,
                       q[l - winSize + 1][l];
       }
 
-      pR[k][l] *= exp_E_ExtLoop(type,
-                                (k > 1) ? S1[k - 1] : -1,
-                                (l < n) ? S1[l + 1] : -1,
-                                pf_params);
+      pR[k][l] *= vrna_exp_E_ext_stem(type,
+                                      (k > 1) ? S1[k - 1] : -1,
+                                      (l < n) ? S1[l + 1] : -1,
+                                      pf_params);
     }
 
     if (hc->matrix_local[k][l - k] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP_ENC) {
       FLT_OR_DBL ppp;
 
-      type_2  = ptype[k][l];
-      type_2  = rtype[type_2];
-
-      if (type_2 == 0)
-        type_2 = 7;
-
+      type_2  = rtype[vrna_get_ptype_window(k, l + k, ptype)];
       ppp     = 0.;
       start_i = k - MAXLOOP - 1;
 
@@ -1196,10 +1165,7 @@ compute_probs(vrna_fold_compound_t        *vc,
             break;
 
           if (hc->matrix_local[i][m - i] & VRNA_CONSTRAINT_CONTEXT_INT_LOOP) {
-            type = ptype[i][m];
-            if (type == 0)
-              type = 7;
-
+            type = vrna_get_ptype_window(i, m + i, ptype);
             if (pR[i][m] > 0) {
               temp = pR[i][m] *
                      exp_E_IntLoop(u1,
@@ -1240,11 +1206,7 @@ compute_probs(vrna_fold_compound_t        *vc,
 
       for (i = MAX2(1, l - winSize + 2); i < k - 1 /* turn */; i++) {
         if (hc->matrix_local[i][m - i] & VRNA_CONSTRAINT_CONTEXT_MB_LOOP) {
-          tt  = ptype[i][m];
-          tt  = rtype[tt];
-          if (tt == 0)
-            tt = 7;
-
+          tt  = rtype[vrna_get_ptype_window(i, m + i, ptype)];
           ppp = pR[i][m] *
                 exp_E_MLstem(tt, S1[m - 1], S1[i + 1], pf_params) *
                 qm[i + 1][k - 1];
@@ -1261,8 +1223,7 @@ compute_probs(vrna_fold_compound_t        *vc,
       prml[m] = prmt;
 
       if (hc->matrix_local[k - 1][m - k + 1] & VRNA_CONSTRAINT_CONTEXT_MB_LOOP) {
-        tt    = ptype[k - 1][m];
-        tt    = rtype[tt];
+        tt    = rtype[vrna_get_ptype_window(k - 1, m + k - 1, ptype)];
         prmt1 = pR[k - 1][m] *
                 expMLclosing *
                 exp_E_MLstem(tt,
@@ -1276,17 +1237,17 @@ compute_probs(vrna_fold_compound_t        *vc,
             prmt1 *= sc->exp_energy_bp_local[k - 1][m - k + 1];
       }
 
-      /* l+1 is unpaired */
-      if (hc->up_ml[l + 1]) {
+      /* k-1 is unpaired */
+      if (hc->up_ml[k - 1]) {
         ppp = prm_l1[m] * expMLbase[1];
 
         if (sc)
           if (sc->exp_energy_up)
-            ppp *= sc->exp_energy_up[l + 1][1];
+            ppp *= sc->exp_energy_up[k - 1][1];
 
         prm_l[m] = ppp + prmt1;
       } else {
-        /* skip configuration where l+1 is unpaired */
+        /* skip configuration where k-1 is unpaired */
         prm_l[m] = prmt1;
       }
 
@@ -1313,15 +1274,16 @@ compute_probs(vrna_fold_compound_t        *vc,
         continue;
 
       if (hc->matrix_local[k][l - k] & VRNA_CONSTRAINT_CONTEXT_MB_LOOP_ENC) {
-        tt = ptype[k][l];
-        if (tt == 0)
-          tt = 7;
+        tt = vrna_get_ptype_window(k, l + k, ptype);
 
         if (options & VRNA_PROBS_WINDOW_UP) {
           double dang;
           /* coefficient for computations of unpairedarrays */
           dang = qb[k][l] *
-                 exp_E_MLstem(tt, S1[k - 1], S1[l + 1], pf_params) *
+                 exp_E_MLstem(tt,
+                              (k > 1) ? S1[k - 1] : -1,
+                              (l < n) ? S1[l + 1] : -1,
+                              pf_params) *
                  scale[2];
 
           for (m = MIN2(k + winSize - 2, n); m >= l + 2; m--) {
@@ -1506,8 +1468,8 @@ compute_stack_probabilities(vrna_fold_compound_t  *vc,
 
   for (j = start + turn + 1; j <= max_j; j++) {
     if ((qb[start][j] * qb[start - 1][(j + 1)]) > 10e-200) {
-      type    = ptype[start - 1][j + 1];
-      type_2  = rtype[(unsigned char)ptype[start][j]];
+      type    = vrna_get_ptype_window(start - 1, j + 1 + start - 1, ptype);
+      type_2  = rtype[vrna_get_ptype_window(start, j + start, ptype)];
       tmp     = qb[start][j] /
                 qb[start - 1][(j + 1)] *
                 exp_E_IntLoop(0,
@@ -1544,7 +1506,7 @@ compute_pU(vrna_fold_compound_t       *vc,
    *  going down to $unpaired, to be unpaired, i.e. a list with entries from 1 to unpaired for
    *  every i, with the probability of a stretch of length x, starting at i-x+1, to be unpaired
    */
-  char              *sequence, **ptype;
+  char              **ptype;
   short             *S1;
   int               startu, i5, j3, len, obp, *rtype, turn, winSize, n, leftmost,
                     rightmost, tt;
@@ -1555,7 +1517,6 @@ compute_pU(vrna_fold_compound_t       *vc,
   vrna_hc_t         *hc;
   vrna_sc_t         *sc;
 
-  sequence      = vc->sequence;
   n             = vc->length;
   winSize       = vc->window_size;
   S1            = vc->sequence_encoding;
@@ -1587,20 +1548,18 @@ compute_pU(vrna_fold_compound_t       *vc,
   QBI = (double *)vrna_alloc((MAX2(ulength, MAXLOOP) + 2) * sizeof(double));
   QBH = (double *)vrna_alloc((MAX2(ulength, MAXLOOP) + 2) * sizeof(double));
 
-  /* first, we will */
-  /* for k<=ulength, pU[k][k]=0, because no bp can enclose it */
+  /*
+   * first, we will
+   * for k<=ulength, pU[k][k]=0, because no bp can enclose it
+   */
 
   /* compute pu[k+ulength][ulength] */
   for (i5 = MAX2(k + ulength - winSize + 1, 1); i5 <= k; i5++) {
     for (j3 = k + ulength + 1; j3 <= MIN2(n, i5 + winSize - 1); j3++) {
       /* Multiloops */
       if (hc->matrix_local[i5][j3 - i5] & VRNA_CONSTRAINT_CONTEXT_MB_LOOP) {
-        tt  = ptype[i5][j3];
-        tt  = rtype[tt];
-        if (tt == 0)
-          tt = 7;
-
-        temp = 0.;
+        tt    = rtype[vrna_get_ptype_window(i5, j3 + i5, ptype)];
+        temp  = 0.;
         /*
          * (.. >-----|..........)
          * i5  j     j+ulength  j3
@@ -1686,7 +1645,7 @@ compute_pU(vrna_fold_compound_t       *vc,
 
   /* Add Interior loop contribution to QBE (and QBI) */
   temp = 0.;
-  for (len = winSize; len >= MAX2(ulength, MAXLOOP); len--)
+  for (len = winSize; len > MAX2(ulength, MAXLOOP); len--)
     temp += QI5[k][len];
 
   for (; len > 0; len--) {
@@ -1718,17 +1677,17 @@ compute_pU(vrna_fold_compound_t       *vc,
   /* add (()()____) type cont. to I3 */
   if (sc && sc->exp_energy_up) {
     for (len = winSize; len >= ulength; len--)
-      if (hc->up_ml[k] >= len) {
+      if (hc->up_ml[k + 1] >= len) {
         temp += q2l[k][len] *
                 expMLbase[len] *
-                sc->exp_energy_up[k][len];
+                sc->exp_energy_up[k + 1][len];
       }
 
     for (; len > 0; len--) {
-      if (hc->up_ml[k] >= len) {
+      if (hc->up_ml[k + 1] >= len) {
         temp += q2l[k][len] *
                 expMLbase[len] *
-                sc->exp_energy_up[k][len];
+                sc->exp_energy_up[k + 1][len];
       }
 
       QBM[len]  += temp;
@@ -1736,12 +1695,12 @@ compute_pU(vrna_fold_compound_t       *vc,
     }
   } else {
     for (len = winSize; len >= ulength; len--)
-      if (hc->up_ml[k] >= len)
+      if (hc->up_ml[k + 1] >= len)
         temp += q2l[k][len] *
                 expMLbase[len];
 
     for (; len > 0; len--) {
-      if (hc->up_ml[k] >= len)
+      if (hc->up_ml[k + 1] >= len)
         temp += q2l[k][len] *
                 expMLbase[len];
 
@@ -1752,7 +1711,7 @@ compute_pU(vrna_fold_compound_t       *vc,
 
   /* add (()___()) */
   for (len = 1; len < ulength; len++) {
-    if (hc->up_ml[k] >= len) {
+    if (hc->up_ml[k + 1] >= len) {
       for (obp = k + len + turn; obp <= MIN2(n, k + winSize - 1); obp++) {
         temp = qmb[k][obp - k - 1] *
                qm[k + len + 1 /*2*/][obp - 1] *
@@ -1760,7 +1719,7 @@ compute_pU(vrna_fold_compound_t       *vc,
 
         if (sc)
           if (sc->exp_energy_up)
-            temp *= sc->exp_energy_up[k][len];
+            temp *= sc->exp_energy_up[k + 1][len];
 
         QBM[len]  += temp;
         QBE[len]  += temp;
@@ -1773,16 +1732,13 @@ compute_pU(vrna_fold_compound_t       *vc,
     if (hc->up_ml[k + 1] >= len) {
       for (obp = k + len + turn + turn; obp <= MIN2(n, k + winSize - 1); obp++) {
         if (hc->matrix_local[k][obp - k] & VRNA_CONSTRAINT_CONTEXT_MB_LOOP) {
-          tt = rtype[ptype[k][obp]];
-          if (tt == 0)
-            tt = 7;
-
-          temp = exp_E_MLstem(tt, S1[obp - 1], S1[k + 1], pf_params) *
-                 scale[2] *
-                 expMLbase[len] *
-                 expMLclosing *
-                 pR[k][obp] *
-                 qm2[k + len + 1][obp - 1]; /* k:obp */
+          tt    = rtype[vrna_get_ptype_window(k, obp + k, ptype)];
+          temp  = exp_E_MLstem(tt, S1[obp - 1], S1[k + 1], pf_params) *
+                  scale[2] *
+                  expMLbase[len] *
+                  expMLclosing *
+                  pR[k][obp] *
+                  qm2[k + len + 1][obp - 1]; /* k:obp */
 
           if (sc) {
             if (sc->exp_energy_up)
@@ -1804,11 +1760,11 @@ compute_pU(vrna_fold_compound_t       *vc,
    * and the unpaired stretch is AT LEAST len long, we start to add that to
    * the old unpaired thingies;
    */
-  for (len = 1; len < MIN2(MAX2(ulength, MAXLOOP), n - k); len++)
+  for (len = 1; len <= MIN2(MAX2(ulength, MAXLOOP), n - k); len++)
     pU[k + len][len] += pU[k + len][len + 1] + QBE[len];
 
   if (options & VRNA_PROBS_WINDOW_UP_SPLIT) {
-    for (len = 1; len < MIN2(MAX2(ulength, MAXLOOP), n - k); len++) {
+    for (len = 1; len <= MIN2(MAX2(ulength, MAXLOOP), n - k); len++) {
       pUH[k + len][len] += pUH[k + len][len + 1] + QBH[len];
       pUM[k + len][len] += pUM[k + len][len + 1] + QBM[len];
       pUI[k + len][len] += pUI[k + len][len + 1] + QBI[len];
@@ -1835,7 +1791,8 @@ compute_pU(vrna_fold_compound_t       *vc,
    */
   for (startu = MIN2(ulength, k); startu > 0; startu--) {
     temp = 0.;
-    if (sc) {
+    /* check whether soft constraint unpaired contributions available */
+    if (sc && sc->exp_energy_up) {
       if (hc->up_ext[k - startu + 1] >= startu) {
         for (i5 = MAX2(1, k - winSize + 2); i5 <= MIN2(k - startu, n - winSize + 1); i5++)
           temp += q[i5][k - startu] *
@@ -1976,6 +1933,7 @@ store_bpp_callback(FLT_OR_DBL *pr,
 }
 
 
+#if 0
 PRIVATE void
 store_stack_prob_callback(FLT_OR_DBL  *pr,
                           int         size,
@@ -2023,6 +1981,9 @@ store_stack_prob_callback(FLT_OR_DBL  *pr,
 }
 
 
+#endif
+
+
 PRIVATE void
 print_pU_callback(double        *pU,
                   int           size,
@@ -2100,11 +2061,31 @@ backward_compat_callback(FLT_OR_DBL   *pr,
 }
 
 
-#ifdef  VRNA_BACKWARD_COMPAT
+#ifndef VRNA_DISABLE_BACKWARD_COMPATIBILITY
+
+/*
+ *###########################################
+ *# deprecated functions below              #
+ *###########################################
+ */
+
+PRIVATE void
+putoutpU_prob_old(double            **pU,
+                  int               length,
+                  int               ulength,
+                  FILE              *fp,
+                  int               energies,
+                  vrna_exp_param_t  *parameters);
+
+
+PRIVATE void
+putoutpU_prob_bin_old(double            **pU,
+                      int               length,
+                      int               ulength,
+                      FILE              *fp,
+                      int               energies,
+                      vrna_exp_param_t  *parameters);
 
-/*###########################################*/
-/*# deprecated functions below              #*/
-/*###########################################*/
 
 PRIVATE vrna_ep_t *
 wrap_pf_foldLP(char             *sequence,
@@ -2117,7 +2098,7 @@ wrap_pf_foldLP(char             *sequence,
                FILE             *spup,
                vrna_exp_param_t *parameters)
 {
-  int                   ulength;
+  int                   ulength, r;
   vrna_fold_compound_t  *vc;
   vrna_md_t             md;
   default_cb_data       data;
@@ -2189,7 +2170,10 @@ wrap_pf_foldLP(char             *sequence,
   if (ulength > 0)
     options |= VRNA_PROBS_WINDOW_UP;
 
-  vrna_probs_window(vc, ulength, options, &backward_compat_callback, (void *)&data);
+  r = vrna_probs_window(vc, ulength, options, &backward_compat_callback, (void *)&data);
+
+  if (!r)
+    return NULL;
 
   if (dpp2 && (*dpp2)) {
     data.stack_prob = (vrna_ep_t *)vrna_realloc(data.stack_prob,
@@ -2294,7 +2278,7 @@ putoutpU_prob(double  **pU,
               int     energies)
 {
   if (backward_compat_compound && backward_compat)
-    putoutpU_prob_par(pU, length, ulength, fp, energies, backward_compat_compound->exp_params);
+    putoutpU_prob_old(pU, length, ulength, fp, energies, backward_compat_compound->exp_params);
   else
     vrna_message_warning("putoutpU_prob: Not doing anything! First, run pfl_fold()!");
 }
@@ -2307,6 +2291,19 @@ putoutpU_prob_par(double            **pU,
                   FILE              *fp,
                   int               energies,
                   vrna_exp_param_t  *parameters)
+{
+  if ((pU) && (fp) && (parameters))
+    putoutpU_prob_old(pU, length, ulength, fp, energies, parameters);
+}
+
+
+PRIVATE void
+putoutpU_prob_old(double            **pU,
+                  int               length,
+                  int               ulength,
+                  FILE              *fp,
+                  int               energies,
+                  vrna_exp_param_t  *parameters)
 {
   /* put out unpaireds */
   int     i, k;
@@ -2351,7 +2348,7 @@ putoutpU_prob_bin(double  **pU,
                   int     energies)
 {
   if (backward_compat_compound && backward_compat)
-    putoutpU_prob_bin_par(pU, length, ulength, fp, energies, backward_compat_compound->exp_params);
+    putoutpU_prob_bin_old(pU, length, ulength, fp, energies, backward_compat_compound->exp_params);
   else
     vrna_message_warning("putoutpU_prob_bin: Not doing anything! First, run pfl_fold()!");
 }
@@ -2364,6 +2361,19 @@ putoutpU_prob_bin_par(double            **pU,
                       FILE              *fp,
                       int               energies,
                       vrna_exp_param_t  *parameters)
+{
+  if ((pU) && (fp) && (parameters))
+    putoutpU_prob_bin_old(pU, length, ulength, fp, energies, parameters);
+}
+
+
+PRIVATE void
+putoutpU_prob_bin_old(double            **pU,
+                      int               length,
+                      int               ulength,
+                      FILE              *fp,
+                      int               energies,
+                      vrna_exp_param_t  *parameters)
 {
   /* put out unpaireds */
   int     i, k, *p;
