@@ -18,21 +18,23 @@
 #include <ctype.h>
 #include <string.h>
 #include <limits.h>
-#include "ViennaRNA/utils.h"
-#include "ViennaRNA/energy_par.h"
-#include "ViennaRNA/energy_const.h" /* defines MINPSCORE */
+
+#include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/params/default.h"
+#include "ViennaRNA/params/constants.h" /* defines MINPSCORE */
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/params.h"
-#include "ViennaRNA/loop_energies.h"
+#include "ViennaRNA/params/basic.h"
+#include "ViennaRNA/loops/all.h"
 #include "ViennaRNA/gquad.h"
 #include "ViennaRNA/ribo.h"
-#include "ViennaRNA/aln_util.h"
-#include "ViennaRNA/mfe.h"
+#include "ViennaRNA/utils/alignments.h"
+#include "ViennaRNA/alphabet.h"
+#include "ViennaRNA/mfe_window.h"
 
 
 #ifdef VRNA_WITH_SVM
-#include "svm.h"
-#include "svm_utils.h"
+#include <svm.h>
+#include "ViennaRNA/utils/svm.h"
 #endif
 
 #ifdef __GNUC__
@@ -85,10 +87,10 @@ make_ptypes(vrna_fold_compound_t  *vc,
             int                   i);
 
 
-PRIVATE char
-*backtrack(vrna_fold_compound_t *vc,
-           int                  start,
-           int                  maxdist);
+PRIVATE char *
+backtrack(vrna_fold_compound_t  *vc,
+          int                   start,
+          int                   maxdist);
 
 
 PRIVATE int
@@ -314,7 +316,7 @@ vrna_mfe_window_zscore_cb(vrna_fold_compound_t            *vc,
 PRIVATE INLINE void
 allocate_dp_matrices(vrna_fold_compound_t *fc)
 {
-  int       i, j, length, maxdist, **c, **fML, n_seq;
+  int       i, j, length, maxdist, **c, **fML;
   vrna_hc_t *hc;
   vrna_sc_t *sc;
 
@@ -335,6 +337,15 @@ allocate_dp_matrices(vrna_fold_compound_t *fc)
       fc->pscore_local[i] = vrna_alloc(sizeof(int) * (maxdist + 5));
   }
 
+  /*
+   *  allocate one more entry for comparative predictions to allow for
+   *  access to i - 1 when processing [i ... maxdist]. This is required
+   *  for (default) hard constraints with noLP option
+   */
+  if (fc->type == VRNA_FC_TYPE_COMPARATIVE)
+    if (length > maxdist + 5)
+      fc->pscore_local[length - maxdist - 5] = vrna_alloc(sizeof(int) * (maxdist + 5));
+
   switch (fc->type) {
     case VRNA_FC_TYPE_SINGLE:
       sc = fc->sc;
@@ -514,6 +525,11 @@ init_constraints(vrna_fold_compound_t *fc,
         make_pscores(fc, i, dm);
         vrna_hc_update(fc, i);
       }
+
+      /* for noLP option */
+      if (length > maxdist + 5)
+        make_pscores(fc, length - maxdist - 5, dm);
+
       break;
   }
 }
@@ -545,11 +561,17 @@ rotate_constraints(vrna_fold_compound_t *fc,
 
     case VRNA_FC_TYPE_COMPARATIVE:
       if (i + maxdist + 4 <= length) {
-        fc->pscore_local[i - 1]           = fc->pscore_local[i + maxdist + 4];
-        fc->pscore_local[i + maxdist + 4] = NULL;
         if (i > 1) {
-          make_pscores(fc, i - 1, dm);
+          fc->pscore_local[i - 2]           = fc->pscore_local[i + maxdist + 4];
+          fc->pscore_local[i + maxdist + 4] = NULL;
+          if (i > 2)
+            make_pscores(fc, i - 2, dm);
+
           vrna_hc_update(fc, i - 1);
+        } else if (i == 1) {
+          free(fc->pscore_local[i - 1]);
+          fc->pscore_local[i - 1]           = fc->pscore_local[i + maxdist + 4];
+          fc->pscore_local[i + maxdist + 4] = NULL;
         }
       }
 
@@ -631,7 +653,7 @@ fill_arrays(vrna_fold_compound_t            *vc,
     /* i,j in [1..length] */
     for (j = i + turn + 1; j <= length && j <= i + maxdist; j++) {
       hc_decompose  = hc->matrix_local[i][j - i];
-      type          = ptype[i][j - i];
+      type          = vrna_get_ptype_window(i, j, ptype);
 
       no_close = (((type == 3) || (type == 4)) && noGUclosure);
 
@@ -674,8 +696,10 @@ fill_arrays(vrna_fold_compound_t            *vc,
         c[i][j - i] = INF;
       }
 
-      /* done with c[i,j], now compute fML[i,j] */
-      /* free ends ? -----------------------------------------*/
+      /*
+       * done with c[i,j], now compute fML[i,j]
+       * free ends ? -----------------------------------------
+       */
       fML[i][j - i] = vrna_E_ml_stems_fast(vc, i, j, Fmi, DMLi);
     } /* for (j...) */
 
@@ -1018,9 +1042,7 @@ repeat1:
 
     switch (vc->type) {
       case VRNA_FC_TYPE_SINGLE:
-        type = ptype[i][j - i];
-        if (type == 0)
-          type = 7;
+        type = vrna_get_ptype_window(i, j, ptype);
 
         no_close = (((type == 3) || (type == 4)) && noGUclosure);
 
@@ -1102,11 +1124,10 @@ fill_arrays_comparative(vrna_fold_compound_t      *fc,
   /* fill "c", "fML" and "f3" arrays and return  optimal energy */
   short **S;
   char **strings, *prev, *ss;
-  int **pscore, i, j, length, energy, turn,
-      n_seq, **c, **fML, *f3, *cc, *cc1, *Fmi,
-      *DMLi, *DMLi1, *DMLi2, maxdist, prev_i, prev_j,
-      prev_en, prev_end, do_backtrack, with_gquad, new_c,
-      psc, stackEnergy, jjj, iii, eee, dangle_model;
+  int **pscore, i, j, length, energy, turn, n_seq, **c,
+      **fML, *f3, *cc, *cc1, *Fmi, *DMLi, *DMLi1, *DMLi2,
+      maxdist, prev_i, prev_j, prev_en, prev_end, with_gquad,
+      new_c, psc, stackEnergy, dangle_model;
   float **dm;
   vrna_mx_mfe_t *matrices;
   vrna_param_t *P;
@@ -1399,8 +1420,10 @@ cov_score(vrna_fold_compound_t  *fc,
   } else {
     for (k = 1, score = 0.; k <= 6; k++) /* ignore pairtype 7 (gap-gap) */
       for (l = k; l <= 6; l++)
-        /* scores for replacements between pairtypes    */
-        /* consistent or compensatory mutations score 1 or 2  */
+        /*
+         * scores for replacements between pairtypes
+         * consistent or compensatory mutations score 1 or 2
+         */
         score += pfreq[k] * pfreq[l] * dm[k][l];
   }
 
@@ -1414,9 +1437,11 @@ make_pscores(vrna_fold_compound_t *fc,
              int                  i,
              float                **dm)
 {
-  /* calculate co-variance bonus for each pair depending on  */
-  /* compensatory/consistent mutations and incompatible seqs */
-  /* should be 0 for conserved pairs, >0 for good pairs      */
+  /*
+   * calculate co-variance bonus for each pair depending on
+   * compensatory/consistent mutations and incompatible seqs
+   * should be 0 for conserved pairs, >0 for good pairs
+   */
   int n, j, **pscore, maxd, turn, noLP;
   vrna_md_t *md;
 
