@@ -21,13 +21,16 @@
 #include "ViennaRNA/fold.h"
 #include "ViennaRNA/cofold.h"
 #include "ViennaRNA/fold_vars.h"
-#include "ViennaRNA/utils.h"
-#include "ViennaRNA/read_epars.h"
+#include "ViennaRNA/utils/basic.h"
+#include "ViennaRNA/utils/strings.h"
+#include "ViennaRNA/params/io.h"
 #include "ViennaRNA/subopt.h"
-#include "ViennaRNA/params.h"
-#include "ViennaRNA/constraints.h"
-#include "ViennaRNA/constraints_SHAPE.h"
-#include "ViennaRNA/file_formats.h"
+#include "ViennaRNA/params/basic.h"
+#include "ViennaRNA/constraints/basic.h"
+#include "ViennaRNA/constraints/SHAPE.h"
+#include "ViennaRNA/io/file_formats.h"
+#include "ViennaRNA/io/utils.h"
+#include "ViennaRNA/commands.h"
 #include "RNAsubopt_cmdl.h"
 #include "gengetopt_helper.h"
 #include "input_id_helpers.h"
@@ -38,6 +41,24 @@ PRIVATE void putoutzuker(FILE                   *output,
                          vrna_subopt_solution_t *zukersolution);
 
 
+struct nr_en_data {
+  FILE                  *output;
+  vrna_fold_compound_t  *fc;
+  double                kT;
+  double                ens_en;
+};
+
+
+PRIVATE void
+print_samples(const char  *structure,
+              void        *data);
+
+
+PRIVATE void
+print_samples_en(const char *structure,
+                 void       *data);
+
+
 int
 main(int  argc,
      char *argv[])
@@ -47,14 +68,15 @@ main(int  argc,
   char                                *rec_sequence, *rec_id,
                                       **rec_rest, *orig_sequence, *constraints_file, *cstruc,
                                       *structure, *shape_file, *shape_method, *shape_conversion,
-                                      *id_prefix, *id_delim, *infile, *outfile, *filename_delim;
+                                      *infile, *outfile, *filename_delim;
   unsigned int                        rec_type, read_opt;
   int                                 i, length, cl, istty, delta, n_back, noconv, dos, zuker,
                                       with_shapes, verbose, enforceConstraints, st_back_en, batch,
-                                      auto_id, id_digits, tofile, filename_full, canonicalBPonly;
-  long int                            seq_number;
+                                      tofile, filename_full, canonicalBPonly, nonRedundant;
   double                              deltap;
   vrna_md_t                           md;
+  dataset_id                          id_control;
+  vrna_cmd_t                          commands;
 
   do_backtrack    = 1;
   delta           = 100;
@@ -65,13 +87,14 @@ main(int  argc,
   cstruc          = structure = NULL;
   verbose         = 0;
   st_back_en      = 0;
-  auto_id         = 0;
   infile          = NULL;
   outfile         = NULL;
   output          = NULL;
   tofile          = 0;
   filename_full   = 0;
   canonicalBPonly = 0;
+  commands        = NULL;
+  nonRedundant    = 0;
 
   set_model_details(&md);
 
@@ -87,12 +110,7 @@ main(int  argc,
     exit(1);
 
   /* parse options for ID manipulation */
-  ggo_get_ID_manipulation(args_info,
-                          auto_id,
-                          id_prefix, "sequence",
-                          id_delim, "_",
-                          id_digits, 4,
-                          seq_number, 1);
+  ggo_get_id_control(args_info, id_control, "Sequence", "sequence", "_", 4, 1);
 
   /* get basic set of model details */
   ggo_get_md_eval(args_info, md);
@@ -100,6 +118,9 @@ main(int  argc,
   ggo_get_md_part(args_info, md);
   ggo_get_circ(args_info, md.circ);
 
+  /* temperature */
+  ggo_get_temperature(args_info, md.temperature);
+
   /* check dangle model */
   if ((md.dangles < 0) || (md.dangles > 3)) {
     vrna_message_warning("required dangle model not implemented, falling back to default dangles=2");
@@ -200,10 +221,12 @@ main(int  argc,
   /* filename sanitize delimiter */
   if (args_info.filename_delim_given)
     filename_delim = strdup(args_info.filename_delim_arg);
+  else if (get_id_delim(id_control))
+    filename_delim = strdup(get_id_delim(id_control));
   else
-    filename_delim = strdup(id_delim);
+    filename_delim = NULL;
 
-  if (isspace(*filename_delim)) {
+  if ((filename_delim) && isspace(*filename_delim)) {
     free(filename_delim);
     filename_delim = NULL;
   }
@@ -212,6 +235,14 @@ main(int  argc,
   if (args_info.filename_full_given)
     filename_full = 1;
 
+  /* non-redundant backtracing */
+  if (args_info.nonRedundant_given)
+    nonRedundant = 1;
+
+  if (args_info.commands_given)
+    commands = vrna_file_commands_read(args_info.commands_arg,
+                                       VRNA_CMD_PARSE_HC | VRNA_CMD_PARSE_SC);
+
   /* free allocated memory of command line data structure */
   RNAsubopt_cmdline_parser_free(&args_info);
 
@@ -278,7 +309,8 @@ main(int  argc,
     }
 
     /* construct the sequence ID */
-    ID_generate(SEQ_ID, rec_id, auto_id, id_prefix, id_delim, id_digits, seq_number, filename_full);
+    set_next_id(&rec_id, id_control);
+    SEQ_ID = fileprefix_from_id(rec_id, id_control, filename_full);
 
     if (tofile) {
       /* prepare the file name */
@@ -323,8 +355,7 @@ main(int  argc,
     /* parse the rest of the current dataset to obtain a structure constraint */
     if (fold_constrained) {
       if (constraints_file) {
-        vrna_constraints_add(vc, constraints_file,
-                             VRNA_OPTION_MFE | ((n_back > 0) ? VRNA_OPTION_PF : 0));
+        vrna_constraints_add(vc, constraints_file, VRNA_OPTION_DEFAULT);
       } else {
         cstruc = NULL;
         int           cp        = -1;
@@ -370,6 +401,11 @@ main(int  argc,
                                  VRNA_OPTION_MFE | ((n_back > 0) ? VRNA_OPTION_PF : 0));
     }
 
+    if (commands)
+      vrna_commands_apply(vc,
+                          commands,
+                          VRNA_CMD_PARSE_HC | VRNA_CMD_PARSE_SC);
+
     if (istty) {
       if (cut_point == -1) {
         vrna_message_info(stdout, "length = %d", length);
@@ -396,7 +432,10 @@ main(int  argc,
 
     /* stochastic backtracking */
     if (n_back > 0) {
-      double mfe, kT, ens_en;
+      double        mfe, kT, ens_en;
+      unsigned int  options = (nonRedundant) ?
+                              VRNA_PBACKTRACK_NON_REDUNDANT :
+                              VRNA_PBACKTRACK_DEFAULT;
 
       if (vc->cutpoint != -1)
         vrna_message_error("Boltzmann sampling for cofolded structures not implemented (yet)!");
@@ -410,21 +449,26 @@ main(int  argc,
       vrna_exp_params_rescale(vc, &mfe);
       /* ignore return value, we are not interested in the free energy */
       ens_en  = vrna_pf(vc, structure);
-      kT      = vc->exp_params->kT;
+      kT      = vc->exp_params->kT / 1000.;
 
-      for (i = 0; i < n_back; i++) {
-        char *s, *e_string = NULL;
-        s = vrna_pbacktrack(vc);
-        if (st_back_en) {
-          double e, prob;
-          e         = vrna_eval_structure(vc, s);
-          prob      = exp((ens_en - e) / kT);
-          e_string  = vrna_strdup_printf(" %6.2f %6g", e, prob);
-        }
+      if (st_back_en) {
+        struct nr_en_data dat;
+        dat.output  = output;
+        dat.fc      = vc;
+        dat.kT      = kT;
+        dat.ens_en  = ens_en;
 
-        print_structure(output, s, e_string);
-        free(s);
-        free(e_string);
+        vrna_pbacktrack_cb(vc,
+                           n_back,
+                           &print_samples_en,
+                           (void *)&dat,
+                           options);
+      } else {
+        vrna_pbacktrack_cb(vc,
+                           n_back,
+                           &print_samples,
+                           (void *)output,
+                           options);
       }
     }
     /* normal subopt */
@@ -501,8 +545,6 @@ main(int  argc,
     if (with_shapes || (constraints_file && (!batch)))
       break;
 
-    ID_number_increase(&seq_number, "Sequence");
-
     /* print user help for the next round if we get input from tty */
     if (istty) {
       if (!zuker)
@@ -525,14 +567,46 @@ main(int  argc,
 
   free(shape_method);
   free(shape_conversion);
-  free(id_prefix);
-  free(id_delim);
   free(filename_delim);
+  vrna_commands_free(commands);
+
+  free_id_data(id_control);
 
   return EXIT_SUCCESS;
 }
 
 
+PRIVATE void
+print_samples(const char  *structure,
+              void        *data)
+{
+  if (structure)
+    print_structure((FILE *)data, structure, NULL);
+}
+
+
+PRIVATE void
+print_samples_en(const char *structure,
+                 void       *data)
+{
+  if (structure) {
+    struct nr_en_data     *d      = (struct nr_en_data *)data;
+    FILE                  *output = d->output;
+    vrna_fold_compound_t  *fc     = d->fc;
+    double                kT      = d->kT;
+    double                ens_en  = d->ens_en;
+
+    double                e         = vrna_eval_structure(fc, structure);
+    double                prob      = exp((ens_en - e) / kT);
+    char                  *e_string = vrna_strdup_printf(" %6.2f %6g", e, prob);
+
+    print_structure(output, structure, e_string);
+
+    free(e_string);
+  }
+}
+
+
 PRIVATE void
 putoutzuker(FILE                    *output,
             vrna_subopt_solution_t  *zukersolution)
